/**
   @file
   @author Shin'ichiro NAKAOKA
*/

#include "ZMPSeq.h"
#include "BodyMotion.h"

using namespace std;
using namespace boost;
using namespace cnoid;

namespace {
    static const string zmpkey("ZMP");
}
    

ZMPSeq::ZMPSeq(int nFrames)
    : Vector3Seq(nFrames)
{
    setSeqContentName(zmpkey);
}


ZMPSeq::ZMPSeq(const ZMPSeq& org)
    : Vector3Seq(org)
{

}


ZMPSeq::ZMPSeq(const Vector3Seq& org)
    : Vector3Seq(org)
{
    setSeqContentName(zmpkey);
}


AbstractSeqPtr ZMPSeq::cloneSeq() const
{
    return make_shared<ZMPSeq>(*this);
}


const std::string& ZMPSeq::key()
{
    return zmpkey;
}


ZMPSeqPtr cnoid::getZMPSeq(const BodyMotion& motion)
{
    return motion.extraSeq<ZMPSeq>(zmpkey);
}


ZMPSeqPtr cnoid::getOrCreateZMPSeq(BodyMotion& motion)
{
    return motion.getOrCreateExtraSeq<ZMPSeq>(zmpkey);
}


void cnoid::clearZMPSeq(BodyMotion& motion)
{
    motion.clearExtraSeq(zmpkey);
}
