/**
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BODYPLUGIN_SCENE_BODY_H_INCLUDED
#define CNOID_BODYPLUGIN_SCENE_BODY_H_INCLUDED

#include <cnoid/SceneViewEditable>
#include <boost/signals/connection.hpp>
#include "exportdecl.h"

namespace cnoid {

    class BodyItem;
    typedef boost::intrusive_ptr<BodyItem> BodyItemPtr;
    class Device;
    class Camera;
    class Light;
    class PointLight;
    class SpotLight;
    class ConnectionSet;
    class SceneBodyImpl;
    
    class CNOID_EXPORT SceneDevice : public SgTransform
    {
    public:
        SceneDevice(Device* device);
        ~SceneDevice();

        template <class DeviceType> DeviceType* device() {
            return static_cast<DeviceType*>(device_);
        }
        template <class DeviceType> const DeviceType* device() const {
            return static_cast<DeviceType*>(device_);
        }
        Device* device() { return device_; }
        const Device* device() const { return device_; }

        void addUpdateConnection(const boost::signals::connection& connection);

      private:
        Device* device_;
        ConnectionSet* connections;
    };
    
    typedef boost::intrusive_ptr<SceneDevice> SceneDevicePtr;
    

    class CNOID_EXPORT SceneBody : public SgTransform, public SceneViewEditable
    {
      public:
        SceneBody(BodyItemPtr bodyItem);

        void updatePositions(bool doNotify);

        virtual bool onKeyPressEvent(const SceneViewEvent& event);
        virtual bool onKeyReleaseEvent(const SceneViewEvent& event);
        virtual bool onButtonPressEvent(const SceneViewEvent& event);
        virtual bool onDoubleClickEvent(const SceneViewEvent& event);
        virtual bool onButtonReleaseEvent(const SceneViewEvent& event);
        virtual bool onPointerMoveEvent(const SceneViewEvent& event);
        virtual void onPointerLeaveEvent(const SceneViewEvent& event);
        virtual bool onScrollEvent(const SceneViewEvent& event);
        virtual void onContextMenuRequest(const SceneViewEvent& event, MenuManager& menuManager);
        virtual void onSceneModeChanged(SceneMode mode);
        virtual bool onUndoRequest();
        virtual bool onRedoRequest();

        // for integrating new device types

        typedef boost::function<SgNode*(SceneDevice* sdev)> DeviceNodeFactory;

        template<class DeviceType>
        static void registerDeviceNodeFactory(const DeviceNodeFactory& factory) {
            registerDeviceNodeFactory_(&typeid(factory), factory);
        }
        
        static void updatePerspectiveCamera(Camera* camera, SgPerspectiveCamera* scamera);
        static void updateLight(Light* light, SgLight* slight);
        static void updatePointLight(PointLight* light, SgPointLight* slight);
        static void updateSpotLight(SpotLight* light, SgSpotLight* slight);

        void setEditable(const bool on){ isEditable_ = on; }
        bool isEditable(){ return isEditable_; }

      protected:
        virtual ~SceneBody();

      private:
        friend class SceneBodyImpl;
        SceneBodyImpl* impl;
        bool isEditable_;

        static void registerDeviceNodeFactory_(const std::type_info* pTypeInfo, const DeviceNodeFactory& factory);
    };
            
    typedef boost::intrusive_ptr<SceneBody> SceneBodyPtr;
}
    
#endif
