/*!
  @file
  @author Shin'ichiro Nakaoka
*/

#include <cnoid/CorbaPlugin>
#include "BodyRTCItem.h"
#include "RTCItem.h"
#include "OpenHRPClockGeneratorItem.h"
#include "ChoreonoidExecutionContext.h"
#include <cnoid/Plugin>
#include <cnoid/ItemManager>
#include <cnoid/MessageView>
#include <QTcpSocket>
#include <boost/thread/thread.hpp>
#include "gettext.h"

using namespace std;
using namespace cnoid;

namespace {

    void rtcManagerMainLoop() {
        RTC::Manager::instance().runManager();
    }
    
    class OpenRTMPlugin : public Plugin
    {
        boost::thread rtcManagerMainLoopThread;
        
    public:
        OpenRTMPlugin() : Plugin("OpenRTM") {
            require("Body");
            require("Corba");
            precede("Corba");
        }
        
        virtual bool initialize() {

            const char* argv[] = {
                "choreonoid",
                "-o", "logger.enable: NO",
                "-o", "manager.shutdown_on_nortcs: NO",
                "-o", "manager.shutdown_auto: NO",
                "-o", "naming.formats: %n.rtc",
#ifdef Q_OS_WIN32
                // To reduce the startup time on Windows
                "-o", "corba.args: -ORBclientCallTimeOutPeriod 100"
#endif
                //"-o", "corba.nameservers: localhost",
                //"-o", "exec_cxt.periodic.type: SynchExtTriggerEC",
                //"-o", "exec_cxt.periodic.rate: 1000000",
                //"-o", "manager.is_master: YES"
                //"-o", "logger.enable: YES",
                //"-o", "logger.file_name: stdout",
                //"-o", "logger.log_level: TRACE",
                //"-o", "corba.args: -ORBendPoint giop:tcp::2809 -ORBpoaUniquePersistentSystemIds 1"
            };

#ifdef Q_OS_WIN32
            int numArgs = 11;
#else
            int numArgs = 9;
#endif

            cnoid::checkOrInvokeCorbaNameServer();

            RTC::Manager* manager = RTC::Manager::init(numArgs, const_cast<char**>(argv));

            if(manager->registerECFactory("ChoreonoidExecutionContext",
                                          RTC::ECCreate<cnoid::ChoreonoidExecutionContext>,
                                          RTC::ECDelete<cnoid::ChoreonoidExecutionContext>)){
                MessageView::instance()->putln(_("ChoreonoidExecutionContext has been registered."));
            }
            
            manager->activateManager();
            
#ifdef Q_OS_WIN32
            omniORB::setClientCallTimeout(0); // reset the global timeout setting?
#endif
                
            if(!cnoid::takeOverCorbaPluginInitialization(manager->getORB())){
                return false;
            }

            BodyRTCItem::initialize(this);
            RTCItem::initialize(this);
            OpenHRPClockGeneratorItem::initialize(this);
            
            VirtualRobotRTC::registerFactory(manager, "VirtualRobot");
            
            rtcManagerMainLoopThread = boost::thread(rtcManagerMainLoop);
            
            return true;
        }

        virtual bool finalize() {

            // delete all the components owned by exisiting BodyRTCItems
            itemManager().detachAllManagedTypeItemsFromRoot();

            RTC::Manager& manager = RTC::Manager::instance();
            manager.shutdown();
            manager.unloadAll();

            return true;
        }
    };
}

CNOID_IMPLEMENT_PLUGIN_ENTRY(OpenRTMPlugin);
