/*!
  @file
  @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_UTIL_TRIANGLE_MESH_SHAPER_H_INCLUDED
#define CNOID_UTIL_TRIANGLE_MESH_SHAPER_H_INCLUDED

#include "VRML.h"
#include <boost/signal.hpp>
#include "exportdecl.h"

namespace cnoid
{
    class TMSImpl;
    
    class CNOID_EXPORT TriangleMeshShaper
    {
      public:

        TriangleMeshShaper();
        ~TriangleMeshShaper();

        void setDivisionNumber(int n);
        void enableNormalGeneration(bool on, bool doOverwrite = false);
        void setMinimumCreaseAngle(double creaseAngle);
        VRMLNodePtr apply(VRMLNodePtr topNode);
        SFNode getOriginalGeometry(VRMLShapePtr shapeNode);
        void defaultTextureMapping(VRMLShape* shapeNode);
        
        boost::signal<void(const std::string& message)> sigMessage;
        
        bool convertBox(VRMLBox* box, VRMLIndexedFaceSetPtr& triangleMesh);

      private:
        TMSImpl* impl;

        void defaultTextureMappingFaceSet(VRMLIndexedFaceSet* triangleMesh);
        void defaultTextureMappingElevationGrid(VRMLElevationGrid* grid, VRMLIndexedFaceSet* triangleMesh);
        void defaultTextureMappingBox(VRMLIndexedFaceSet* triangleMesh);
        void defaultTextureMappingCone(VRMLIndexedFaceSet* triangleMesh);
        void defaultTextureMappingCylinder(VRMLIndexedFaceSet* triangleMesh);
        void defaultTextureMappingSphere(VRMLIndexedFaceSet* triangleMesh, float radius);
        void defaultTextureMappingExtrusion(VRMLIndexedFaceSet* triangleMesh, VRMLExtrusion* extrusion);
        int faceofBox(SFVec3s* point);
        int findPoint(MFVec2s& points, SFVec2s& target);
        float calcangle(SFVec3s& point);
    };

    enum { LEFT, TOP, FRONT, BOTTOM, RIGHT, BACK };

};

#endif

